#!/bin/bash -vx

# this script convolves all images to the same seeing

#$1: directory
#$2: image list (one file per line)

. progs.ini 

${P_GAWK} '($1 !~ /BACK_TYPE/ && $1 !~ /BACK_VALUE/) {print}' \
    ${DATACONF}/postcoadd.conf.sex > ${TEMPDIR}/postcoadd.conf.sex_$$
POSTCOADD=${TEMPDIR}/postcoadd.conf.sex_$$

dir=/$1/seeing/
data=/$1/


###########################################
# extent of the convolution kernel

test -f ${TEMPDIR}/seeingconv.dat_$$ && rm ${TEMPDIR}/seeingconv.dat_$$

cat $1/$2 |\
{
  while read file
  do
    ${P_GAWK} '($1 == "'${file}'")' ${dir}/seeing.dat >> ${TEMPDIR}/seeingconv.dat_$$
  done
}


${P_GAWK} 'BEGIN{s=0}{r=$2/sqrt(4.*log(4.)); if (r>s) s=r; data[NR]=r} END {
     for (i=1;i<=NR;i++) {print sqrt(s*s-data[i]*data[i])}
     }' ${TEMPDIR}/seeingconv.dat_$$ > ${TEMPDIR}/convkernelsize_$$

seeing_aim=`${P_GAWK} 'BEGIN{s=0}{if ($2>s) s=$2} END {print s}' \
    ${TEMPDIR}/seeingconv.dat_$$`

###########################################
# convolve


if [ ! -d ${data}/unconvolved ]; then
    mkdir ${data}/unconvolved
fi

if [ -f ${dir}/seeingconvolved.dat ]; then
    rm ${dir}/seeingconvolved.dat
fi

i=1

cd $1

cat $2 |\
{
  while read file
  do
    BASE=`basename ${file} .fits`
    size=`${P_GAWK} '(NR=='${i}')' ${TEMPDIR}/convkernelsize_$$`
    seeing[i]=`${P_GAWK} '(NR=='${i}') {print $2}' ${dir}/seeing.dat`
    if [ ! -f ${data}/unconvolved/${file} ]; then
	mv /${data}/${file} ${data}/unconvolved
    fi

    if [ ! "${size}" = "0" ]; then
	
	loop=0
	convcrit="unconverged"
	while [ ${convcrit} != "converged" ] && [ ${loop} -le 4 ]
	do
	  # convolve
	  ${P_FITSGAUSS} -i ${data}/unconvolved/${file} \
	      -o /${data}/${file} -s ${size}
	  
	  # crosscheck the seeing in the result
          weightstring=""
          if [ -f /${data}/${BASE}.weight.fits ]; then
	      weightstring="-WEIGHT_IMAGE /${data}/${BASE}.weight.fits -WEIGHT_TYPE MAP_WEIGHT"
          fi
          ${P_SEX} -c ${POSTCOADD} \
	      /${data}/${file} ${weightstring} \
	      -CATALOG_NAME ${TEMPDIR}/${BASE}_convcheck.cat_$$ \
	      -CATALOG_TYPE FITS_LDAC \
	      -FILTER Y \
	      -DETECT_MINAREA 2 \
	      -DETECT_THRESH 2 \
	      -ANALYSIS_THRESH 2 \
	      -MAG_ZEROPOINT 20 \
	      -DEBLEND_MINCONT 0.0001
	  
	  ${P_LDACTOASC} -i ${TEMPDIR}/${BASE}_convcheck.cat_$$ \
	      -t LDAC_OBJECTS -k FLUX_RADIUS -b > \
	      ${TEMPDIR}/${BASE}_convcheck.ascii_$$
	  
	  seeing_conv=`${P_GETSTATS} -i ${TEMPDIR}/${BASE}_convcheck.ascii_$$ -m mode | \
	      ${P_GAWK} '{print $1*2.}'`
	  
	  convcrit=`echo ${seeing_aim} ${seeing_conv} | \
	      ${P_GAWK} '{diff=$2-$1} END {
                          if (diff < 0.1 && diff > -0.1) print "converged"; else print "unvonverged"}'`

	  if [ ${convcrit} != "converged" ]; then
	      size=`echo ${seeing[i]} ${seeing_aim} ${seeing_conv} ${size} | \
	          ${P_GAWK} '{print ($2-$1)/($3-$1)*$4}'`
	      loop=$((${loop}+1))
	  fi
	done
	echo ${file} ${seeing[i]} ${seeing_conv} >> ${dir}/seeingconvolved.dat
    else
	cp ${data}/unconvolved/${file} /${data}/${file}
	echo ${file} ${seeing[i]} ${seeing_aim} >> ${dir}/seeingconvolved.dat
    fi

    i=$((${i}+1))
  done
}

\rm /$1/$2
\rm ${TEMPDIR}/*_$$
