#!/bin/bash -xv

# this script calculates and applies colour correction factors
# for red-, green- and blue-filtered images.

# $1: this directory
# $2: dir with the images
# $3: blue image
# $4: green image
# $5: red image

cd $1

. progs.ini

${P_GAWK} '($1 !~ /BACK_TYPE/ && $1 !~ /BACK_VALUE/) {print}' \
    ${DATACONF}/postcoadd.conf.sex > ${TEMPDIR}/postcoadd.conf.sex_$$
POSTCOADD=${TEMPDIR}/postcoadd.conf.sex_$$

V_COLORCORR_B=$3
V_COLORCORR_G=$4
V_COLORCORR_R=$5

CALIBDIR=$2/AVGWHITE_calibration

# CALIBDIR is created from within the GUI itself

test -f ${CALIBDIR}/AVGWHITE_calibration.dat && rm ${CALIBDIR}/AVGWHITE_calibration.dat

# make the detection image
${P_IC} '%1 %2 + %3 +' /$2/$3 /$2/$4 /$2/$5 > /${CALIBDIR}/detect_image.fits

# make the detection weight (if weights are present)
base=`basename ${V_COLORCORR_B} .fits`
BW=/$2/${base}.weight.fits
base=`basename ${V_COLORCORR_G} .fits`
GW=/$2/${base}.weight.fits
base=`basename ${V_COLORCORR_R} .fits`
RW=/$2/${base}.weight.fits

if [ -f ${BW} ] && [ -f ${GW} ] && [ -f ${RW} ]; then
    ${P_IC} '%1 %2 + %3 +' ${BW} ${GW} ${RW} > /${CALIBDIR}/detect_image.weight.fits
fi

# create object catalogues
images="${V_COLORCORR_B} ${V_COLORCORR_G} ${V_COLORCORR_R}" 

for image in ${images}
do
    base=`basename ${image} .fits`

    if [ -f "/${CALIBDIR}/${base}.cat" ]; then
	continue
    fi

    imagestring="/${CALIBDIR}/detect_image.fits,/$2/${image}"
    weightstring=""
    if [ -f /${CALIBDIR}/detect_image.weight.fits ]; then
	weightstring="-WEIGHT_IMAGE /${CALIBDIR}/detect_image.weight.fits,/$2/${base}.weight.fits"
        weightstring=${weightstring}" -WEIGHT_TYPE MAP_WEIGHT,MAP_WEIGHT"
    fi

    # saturation value = conservative 0.85*max(image)
    satlevel=`${P_STATS} < /$2/${image} | grep max | ${P_GAWK} '{print 0.85*$6}'`

    ${P_SEX} -c ${POSTCOADD} \
	/$2/${image} ${weightstring} \
	-CATALOG_NAME ${TEMPDIR}/${base}.cat \
	-CATALOG_TYPE FITS_LDAC \
	-FILTER Y \
	-DETECT_MINAREA 5 \
	-DETECT_THRESH 5 \
	-ANALYSIS_THRESH 5 \
	-MAG_ZEROPOINT 20 \
	-DEBLEND_MINCONT 0.0001 \
	-NTHREADS ${NPARA} \
	-SATUR_LEVEL ${satlevel}
    
    ${P_LDACCONV} -i ${TEMPDIR}/${base}.cat \
	-o ${TEMPDIR}/${base}_ldac.cat -b 1 -c "sex" -f R
    
    ${P_LDACADDKEY} -i ${TEMPDIR}/${base}_ldac.cat \
	-o ${TEMPDIR}/${base}_ldac_tmp.cat \
	-t OBJECTS -k nu 1.0 FLOAT ""
    
    ${P_LDACCALC} -i ${TEMPDIR}/${base}_ldac_tmp.cat \
	-o ${TEMPDIR}/tmp1.cat_$$ -t OBJECTS \
	-c "(FLUX_RADIUS);" -n "rg" "" -k FLOAT \
	-c "(Xpos-1.0);" -n "x" "" -k LONG \
	-c "(Ypos-1.0);" -n "y" "" -k LONG \
	-c "(Xpos);" -n "xbad" "" -k FLOAT \
	-c "(Ypos);" -n "ybad" "" -k FLOAT
    
    ${P_LDACTOASC} -i ${TEMPDIR}/tmp1.cat_$$ -b -t FIELDS \
	-k FITSFILE SEXBKGND SEXBKDEV -s > ${TEMPDIR}/tmp.asc_$$
    
    # create a config file for asctoldac on the fly
    
    echo "VERBOSE = DEBUG"       >  ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = IMAGE"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = STRING"    >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = STRING"    >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 128'       >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = SKY_MODE"  >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 1'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = SKY_SIGMA" >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 1'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    
    ${P_ASCTOLDAC} -i ${TEMPDIR}/tmp.asc_$$ \
	-c ${TEMPDIR}/asctoldac_tmp.conf_$$ -t HFINDPEAKS \
	-o ${TEMPDIR}/hfind.cat_$$ -b 1 -n "KSB"
    
    # now transfer the HFINDPEAKS table to the SEX catalog
    
    ${P_LDACADDTAB} -i ${TEMPDIR}/tmp1.cat_$$ \
	-o ${TEMPDIR}/tmp2.cat_$$ \
	-p ${TEMPDIR}/hfind.cat_$$ \
	-t HFINDPEAKS
    
    ${P_LDACFILTER} -i ${TEMPDIR}/tmp2.cat_$$ \
	-o /${CALIBDIR}/${base}.cat \
	-c "((((rg>0.0)AND(Flag=0))AND(MAG_AUTO<99))AND(MAGERR_AUTO<0.15));"
done

# the individual catalogues

bcat=`basename ${V_COLORCORR_B} .fits`.cat
vcat=`basename ${V_COLORCORR_G} .fits`.cat
rcat=`basename ${V_COLORCORR_R} .fits`.cat

# second, associate and merge the object catalogues

${P_ASSOCIATE} -i ${CALIBDIR}/${bcat} ${CALIBDIR}/${vcat} ${CALIBDIR}/${rcat} \
    -o ${TEMPDIR}/B.cat_$$ ${TEMPDIR}/G.cat_$$ ${TEMPDIR}/R.cat_$$ \
    -t OBJECTS -c ${DATACONF}/colcorr_assoc.conf -INTER_COLOR_TOL 0.5

${P_MAKESSC} -i ${TEMPDIR}/B.cat_$$ ${TEMPDIR}/G.cat_$$ ${TEMPDIR}/R.cat_$$ \
    -o ${TEMPDIR}/RGB.cat_$$ -t OBJECTS -c ${DATACONF}/colcorr_avgwhite_make_ssc.pairs

${P_LDACFILTER} -i ${TEMPDIR}/RGB.cat_$$ -t PSSC -o ${TEMPDIR}/RGB_allsources.cat_$$ \
    -c "(((MAGAUTO_B>0.)AND(MAGAUTO_G>0.))AND(MAGAUTO_R>0.));"

cp ${TEMPDIR}/RGB.cat_$$ ${CALIBDIR}/RGB_unfiltered.cat

# obtain the colour correction factors with respect to G_band

base_b=`basename ${V_COLORCORR_B} .fits`
base_v=`basename ${V_COLORCORR_G} .fits`
base_r=`basename ${V_COLORCORR_R} .fits`

# this is the basis for the calculation of the correction factors
${P_LDACTOASC} -i ${TEMPDIR}/RGB_allsources.cat_$$ -t PSSC \
    -k MAGAUTO_B MAGAUTO_G MAGAUTO_R -b > ${TEMPDIR}/colcorr_ascii.cat_$$

${P_LDACCALC} -i ${TEMPDIR}/RGB_allsources.cat_$$ -t PSSC \
    -c "(MAGAUTO_B-MAGAUTO_G);" -n B-G "" -k FLOAT \
    -c "(MAGAUTO_G-MAGAUTO_R);" -n G-R "" -k FLOAT \
    -o ${CALIBDIR}/RGB_allsources.cat

${P_LDACTOSKYCAT} -i ${CALIBDIR}/RGB_allsources.cat -t PSSC \
    -k SeqNr Ra_G Dec_G MAGAUTO_G \
    -l id_col SeqNr ra_col Ra_G dec_col \
    Dec_G mag_col MAGAUTO_G \
    -s > /${CALIBDIR}/AVGWHITE_sources.skycat

# if no sources are matched ...
nobj=`wc ${TEMPDIR}/colcorr_ascii.cat_$$ | ${P_GAWK} '{print $1}'`
if [ "${nobj}" = "0" ]; then
    echo 0           >  ${CALIBDIR}/AVGWHITE_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/AVGWHITE_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/AVGWHITE_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/AVGWHITE_calibration.dat
    exit
fi

# obtain the colours in units of fluxes
${P_GAWK} '{print 10^(0.4*($1-$2)), 1.0, 10^(0.4*($3-$2))}' \
    ${TEMPDIR}/colcorr_ascii.cat_$$ >> ${TEMPDIR}/colcorr_ascii_fluxes.cat_tmp1_$$

${P_SORT} -n -k 1 ${TEMPDIR}/colcorr_ascii_fluxes.cat_tmp1_$$ >\
    ${TEMPDIR}/colcorr_ascii_fluxes.cat_$$
    
# obtain the average corrections factors and their rms
${P_GAWK} 'BEGIN {b=0;r=0} {b=b+$1; r=r+$3; barr[NR]=$1; rarr[NR]=$3} 
       END {meanb=b/NR
            meanr=r/NR
            rmsb=0
            rmsr=0
            for (i=1;i<=NR;i++) {
                rmsb=rmsb+(barr[i]-meanb)^2
                rmsr=rmsr+(rarr[i]-meanr)^2
            }
            if (NR>1) {
              rmsb=sqrt(1/(NR-1)*rmsb)/sqrt(NR)
              rmsr=sqrt(1/(NR-1)*rmsr)/sqrt(NR)
            }
            if (NR==1) {
              rmsb=0.0
              rmsr=0.0
            }
            print NR
            printf "%2.3f %2.3f\n%2.3f %2.3f\n%2.3f %2.3f\n", meanb, rmsb, 1.0, 0.0, meanr, rmsr}' \
		${TEMPDIR}/colcorr_ascii_fluxes.cat_$$ >> \
		${CALIBDIR}/AVGWHITE_calibration.dat
