#!/bin/bash -xv

# this script calculates and applies colour correction factors
# for red-, green- and blue-filtered images.

# $1: this directory
# $2: dir with the images
# $3: blue image
# $4: green image
# $5: red image
# $6: B-V min
# $7: B-V max
# $8: V-R min
# $9: V-R max
# $10: NOMAD or SDSS
# $11: server

cd $1

DIR=$1

. progs.ini

. bash.include

${P_GAWK} '($1 !~ /BACK_TYPE/ && $1 !~ /BACK_VALUE/) {print}' \
    ${DATACONF}/postcoadd.conf.sex > ${TEMPDIR}/postcoadd.conf.sex_$$
POSTCOADD=${TEMPDIR}/postcoadd.conf.sex_$$

V_COLORCORR_B=$3
V_COLORCORR_G=$4
V_COLORCORR_R=$5

BVMIN=$6
BVMAX=$7
VRMIN=$8
VRMAX=$9
REFCAT=${10}
SERVER=${11}

# get the NOMAD1 or SDSS catalogue
# automatically obtain data (for amateur astronomy colour calibration)

cd11=`get_key $2/$3 CD1_1`
cd12=`get_key $2/$3 CD1_2`
if [ "${cd11}"_A = "_A" ]; then
    cd11=`${P_CDELT2CD} -i $2/$3 | ${P_GAWK} '{print $1}'`
    cd12=`${P_CDELT2CD} -i $2/$3 | ${P_GAWK} '{print $2}'`
fi
pixscale=`echo $cd11 $cd12 | ${P_GAWK} '{print sqrt($1*$1+$2*$2)}'`
naxis1=`${P_DFITS} $2/$3 | ${P_FITSORT} NAXIS1 | ${P_GAWK} '(NR!=1) {print $2}'`
naxis2=`${P_DFITS} $2/$3 | ${P_FITSORT} NAXIS2 | ${P_GAWK} '(NR!=1) {print $2}'`
radius=`echo $naxis1 $naxis2 $pixscale | ${P_GAWK} '{print sqrt($1*$1+$2*$2)*$3*60*0.5}'`
xcen=`echo $naxis1 | ${P_GAWK} '{print $1/2.}'`
ycen=`echo $naxis2 | ${P_GAWK} '{print $1/2.}'`
center_radec=`${P_XY2SKY} -d $2/$3 ${xcen} ${ycen} |\
              ${P_GAWK} '{if ($2<=0.) {print $1,$2} else {print $1, "+"$2}}'`

CALIBDIR=$2/PHOTCAT_calibration

# CALIBDIR is created from within the GUI itself

test -f ${CALIBDIR}/PHOTCAT_calibration.dat && rm ${CALIBDIR}/PHOTCAT_calibration.dat

# check if the connection can be established
${P_FINDNOMAD1} -c $center_radec -m 10 >& ${TEMPDIR}/testconnection
connection="_A"
connection=`${P_GAWK} '($0~/Network is unreachable/) {print "no"}' ${TEMPDIR}/testconnection`
if [ "${connection}" = "no" ]; then
    echo "unconnected" > ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    exit
fi

ra=`echo ${center_radec} | ${P_GAWK} '{print $1}'`
dec=`echo ${center_radec} | ${P_GAWK} '{print $2}'`

if [ "${REFCAT}" = "NOMAD" ]; then
    ${P_VIZQUERY} -wget -site=${SERVER} -mime=csv \
	-out.max=1000000 -c.rm=${radius} -c="${ra} ${dec}" \
	-out=_RAJ -out=_DEJ -out="Bmag Vmag Rmag" -source=NOMAD1 \
	> ${TEMPDIR}/nomad_colcorr.cat_$$
    ${P_GAWK} '($0 !~ /#/)' ${TEMPDIR}/nomad_colcorr.cat_$$ | \
	${P_GAWK} 'BEGIN{FS=";"} (NR>4) {print $1, $2, $3, $4, $5}' | \
	${P_GAWK} '(NF==5)' > ${TEMPDIR}/colorref_full_$$
fi
if [ "${REFCAT}" = "SDSS" ]; then
#    ${P_VIZQUERY} -wget -site=${SERVER} -mime=csv \
#	-out.max=1000000 -c.rm=${radius} -c="${ra} ${dec}" \
#	-out=_RAJ -out=_DEJ -out=umag -out=gmag -out=rmag -source=SDSS-DR7 \
#	> ${TEMPDIR}/sdss_colcorr.cat_$$
#    ${P_GAWK} '($0 !~ /#/)' ${TEMPDIR}/sdss_colcorr.cat_$$ | \
#	${P_GAWK} 'BEGIN{FS=";"} (NR>4) {print $1, $2, $3, $4, $5}' | \
#	${P_GAWK} '(NF==5)' > ${TEMPDIR}/colorref_full_$$


    ramin=`echo ${ra} ${radius} ${dec} | \
	${P_GAWK} '{ramin=$1-$2/60./cos($3*3.14159/180.); if (ramin<0) {ramin=ramin+360.}} END {print ramin}'`
    ramax=`echo ${ra} ${radius} ${dec} | \
	${P_GAWK} '{ramax=$1+$2/60./cos($3*3.14159/180.); if (ramax>360) {ramax=ramax-360.}} END {print ramax}'`
    decmin=`echo ${dec} ${radius} | ${P_GAWK} '{decmin=$1-$2/60.; if (decmin<-90) {decmin=-90}} END {print decmin}'`
    decmax=`echo ${dec} ${radius} | ${P_GAWK} '{decmax=$1+$2/60.; if (decmax>90) {decmax=90}} END {print decmax}'`

    python ${DIR}/sqlcl.py -l -f csv -q "SELECT ra,dec,u,g,r FROM PhotoObj WHERE ra BETWEEN ${ramin} AND ${ramax} AND dec BETWEEN ${decmin} AND ${decmax} AND clean = 1 AND u > 1 AND g > 1 AND r > 1" | sed 's/,/ /g' > ${TEMPDIR}/colorref_full_$$
fi

nref=`wc ${TEMPDIR}/colorref_full_$$ | ${P_GAWK} '{print $1}'`

echo "NUMBER OF REFERENCE SOURCES DOWNLOADED: " ${nref}

if [ ${nref} -eq 0 ]; then
    echo 0           >  ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    exit;
fi

# prepare the association
${P_GAWK} '{print NR, $1, $2, $3, $4, $5}' ${TEMPDIR}/colorref_full_$$ > \
    ${TEMPDIR}/skc_colorref_tmp2_full_$$

# make a config file for the full nomad catalogue
echo VERBOSE = DEBUG         >  ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = SeqNr       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = LONG        >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = INT         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = ALPHA_J2000 >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = DOUBLE      >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = "deg"        >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = DELTA_J2000 >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = DOUBLE      >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = "deg"        >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = REF_MAG_B   >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = REF_MAG_V   >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_NAME  = REF_MAG_R   >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_TTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_HTYPE = FLOAT       >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_COMM = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_UNIT = '""'         >> ${TEMPDIR}/colorref_a2l_full.conf_$$
echo COL_DEPTH = 1           >> ${TEMPDIR}/colorref_a2l_full.conf_$$

# continue with the association

${P_ASCTOLDAC} -a ${TEMPDIR}/skc_colorref_tmp2_full_$$ \
    -o /${CALIBDIR}/photref.cat \
    -t OBJECTS -c ${TEMPDIR}/colorref_a2l_full.conf_$$

# make the detection image
${P_IC} '%1 %2 + %3 +' /$2/$3 /$2/$4 /$2/$5 > /${CALIBDIR}/detect_image.fits

# make the detection weight (if weights are present)
base=`basename ${V_COLORCORR_B} .fits`
BW=/$2/${base}.weight.fits
base=`basename ${V_COLORCORR_G} .fits`
GW=/$2/${base}.weight.fits
base=`basename ${V_COLORCORR_R} .fits`
RW=/$2/${base}.weight.fits

if [ -f ${BW} ] && [ -f ${GW} ] && [ -f ${RW} ]; then
    ${P_IC} '%1 %2 + %3 +' ${BW} ${GW} ${RW} > /${CALIBDIR}/detect_image.weight.fits
fi

# create object catalogues
images="${V_COLORCORR_B} ${V_COLORCORR_G} ${V_COLORCORR_R}" 

for image in ${images}
do
    base=`basename ${image} .fits`

    # if the catalog exists already, then skip object detection
    if [ -f "/${CALIBDIR}/${base}.cat" ]; then
	continue
    fi

    imagestring="/${CALIBDIR}/detect_image.fits,/$2/${image}"
    weightstring=""
    if [ -f /${CALIBDIR}/detect_image.weight.fits ]; then
	weightstring="-WEIGHT_IMAGE /${CALIBDIR}/detect_image.weight.fits,/$2/${base}.weight.fits"
        weightstring=${weightstring}" -WEIGHT_TYPE MAP_WEIGHT,MAP_WEIGHT"
    fi

    # saturation value = conservative 0.85*max(image)
    satlevel=`${P_STATS} < /$2/${image} | grep max | ${P_GAWK} '{print 0.85*$6}'`

    ${P_SEX} -c ${POSTCOADD} \
	/$2/${image} ${weightstring} \
	-CATALOG_NAME ${TEMPDIR}/${base}.cat \
	-CATALOG_TYPE FITS_LDAC \
	-FILTER Y \
	-DETECT_MINAREA 5 \
	-DETECT_THRESH 5 \
	-ANALYSIS_THRESH 5 \
	-MAG_ZEROPOINT 20 \
	-DEBLEND_MINCONT 0.0001 \
    	-NTHREADS ${NPARA} \
	-SATUR_LEVEL ${satlevel}

    ${P_LDACCONV} -i ${TEMPDIR}/${base}.cat \
	-o ${TEMPDIR}/${base}_ldac.cat -b 1 -c "sex" -f R
    
    ${P_LDACADDKEY} -i ${TEMPDIR}/${base}_ldac.cat \
	-o ${TEMPDIR}/${base}_ldac_tmp.cat \
	-t OBJECTS -k nu 1.0 FLOAT ""
    
    ${P_LDACCALC} -i ${TEMPDIR}/${base}_ldac_tmp.cat \
	-o ${TEMPDIR}/tmp1.cat_$$ -t OBJECTS \
	-c "(FLUX_RADIUS);" -n "rg" "" -k FLOAT \
	-c "(Xpos-1.0);" -n "x" "" -k LONG \
	-c "(Ypos-1.0);" -n "y" "" -k LONG \
	-c "(Xpos);" -n "xbad" "" -k FLOAT \
	-c "(Ypos);" -n "ybad" "" -k FLOAT
    
    ${P_LDACTOASC} -i ${TEMPDIR}/tmp1.cat_$$ -b -t FIELDS \
	-k FITSFILE SEXBKGND SEXBKDEV -s > ${TEMPDIR}/tmp.asc_$$
    
    # create a config file for asctoldac on the fly
    
    echo "VERBOSE = DEBUG"       >  ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = IMAGE"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = STRING"    >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = STRING"    >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 128'       >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = SKY_MODE"  >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 1'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_NAME  = SKY_SIGMA" >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_TTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo "COL_HTYPE = FLOAT"     >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_COMM = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_UNIT = ""'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    echo 'COL_DEPTH = 1'         >> ${TEMPDIR}/asctoldac_tmp.conf_$$
    
    ${P_ASCTOLDAC} -i ${TEMPDIR}/tmp.asc_$$ \
	-c ${TEMPDIR}/asctoldac_tmp.conf_$$ -t HFINDPEAKS \
	-o ${TEMPDIR}/hfind.cat_$$ -b 1 -n "KSB"
    
    # now transfer the HFINDPEAKS table to the SEX catalog
    
    ${P_LDACADDTAB} -i ${TEMPDIR}/tmp1.cat_$$ \
	-o ${TEMPDIR}/tmp2.cat_$$ \
	-p ${TEMPDIR}/hfind.cat_$$ \
	-t HFINDPEAKS
    
    ${P_LDACFILTER} -i ${TEMPDIR}/tmp2.cat_$$ \
	-o /${CALIBDIR}/${base}.cat \
	-c "((((rg>0.0)AND(Flag=0))AND(MAG_AUTO<99))AND(MAGERR_AUTO<0.15));"

    ${P_LDACTOSKYCAT} -i /${CALIBDIR}/${base}.cat -t OBJECTS \
	-k SeqNr ALPHA_J2000 DELTA_J2000 MAG_AUTO \
	-l id_col SeqNr ra_col ALPHA_J2000 dec_col \
	DELTA_J2000 mag_col MAG_AUTO \
	-s > /${CALIBDIR}/${base}.skycat
done

# the individual catalogues

bcat=`basename ${V_COLORCORR_B} .fits`.cat
vcat=`basename ${V_COLORCORR_G} .fits`.cat
rcat=`basename ${V_COLORCORR_R} .fits`.cat

# reformat the reference catalogue so that it would run through associate

${P_LDACDELTAB} -i /${CALIBDIR}/photref.cat -o ${TEMPDIR}/cc1_$$ -t FIELDS
${P_LDACADDTAB} -i ${TEMPDIR}/cc1_$$ -o ${TEMPDIR}/cc2_$$ -p ${CALIBDIR}/${bcat} -t FIELDS
${P_LDACADDKEY} -i ${TEMPDIR}/cc2_$$ -o ${TEMPDIR}/cc3_$$ -t OBJECTS -k Flag 0 SHORT ""
${P_LDACADDKEY} -i ${TEMPDIR}/cc3_$$ -o ${TEMPDIR}/cc4_$$ -t OBJECTS -k A_WORLD 0.00027778 FLOAT ""
${P_LDACADDKEY} -i ${TEMPDIR}/cc4_$$ -o ${TEMPDIR}/cc5_$$ -t OBJECTS -k B_WORLD 0.00027778 FLOAT ""
${P_LDACADDKEY} -i ${TEMPDIR}/cc5_$$ -o ${TEMPDIR}/theli_colorref_full.cat_$$ \
    -t OBJECTS -k THETA_J2000 0.0 FLOAT ""

# second, associate and merge the object and the reference catalogues

${P_ASSOCIATE} -i ${CALIBDIR}/${bcat} ${CALIBDIR}/${vcat} ${CALIBDIR}/${rcat} \
    ${TEMPDIR}/theli_colorref_full.cat_$$ \
    -o ${TEMPDIR}/B.cat_$$ ${TEMPDIR}/G.cat_$$ ${TEMPDIR}/R.cat_$$ ${TEMPDIR}/colorref_tmp_$$ \
    -t OBJECTS -c ${DATACONF}/colcorr_assoc.conf -INTER_COLOR_TOL 0.5

${P_MAKESSC} -i ${TEMPDIR}/B.cat_$$ ${TEMPDIR}/G.cat_$$ \
    ${TEMPDIR}/R.cat_$$ ${TEMPDIR}/colorref_tmp_$$ \
    -o ${TEMPDIR}/RGBcolorref.cat_$$ -t OBJECTS -c ${DATACONF}/colcorr_photcat_make_ssc.pairs

${P_LDACFILTER} -i ${TEMPDIR}/RGBcolorref.cat_$$ -t PSSC -o ${TEMPDIR}/RGBcolorref_allsources.cat_$$ \
    -c "((((MAGAUTO_B>0.)AND(MAGAUTO_G>0.))AND(MAGAUTO_R>0.))AND(REF_MAG_B>0.));"

cp ${TEMPDIR}/RGBcolorref.cat_$$ ${CALIBDIR}/RGBcolorref_unfiltered.cat

# obtain the colour correction factors with respect to G_band

base_b=`basename ${V_COLORCORR_B} .fits`
base_v=`basename ${V_COLORCORR_G} .fits`
base_r=`basename ${V_COLORCORR_R} .fits`

# this is the basis for the calculation of the correction factors
# if V-R constraints are given
if [ "${VRMIN}" != "null" ] && [ "${VRMAX}" != "null" ]; then
    ${P_LDACTOASC} -i ${TEMPDIR}/RGBcolorref_allsources.cat_$$ -t PSSC \
	-k MAGAUTO_B MAGAUTO_G MAGAUTO_R REF_MAG_B REF_MAG_V REF_MAG_R -b |\
	${P_GAWK} '($4-$5<'${BVMAX}' && $4-$5>'${BVMIN}' && $5-$6<'${VRMAX}' && $5-$6>'${VRMIN}') {print $1, $2, $3}' \
	> ${TEMPDIR}/colcorr_ascii.cat_$$
fi

# if no V-R constraints are given
if [ "${VRMIN}" = "null" ] || [ "${VRMAX}" = "null" ]; then
    ${P_LDACTOASC} -i ${TEMPDIR}/RGBcolorref_allsources.cat_$$ -t PSSC \
	-k MAGAUTO_B MAGAUTO_G MAGAUTO_R REF_MAG_B REF_MAG_V REF_MAG_R -b |\
	${P_GAWK} '($4-$5<'${BVMAX}' && $4-$5>'${BVMIN}') {print $1, $2, $3}' \
	> ${TEMPDIR}/colcorr_ascii.cat_$$
fi

# this is a catalog with all the matched nomad sources
${P_LDACCALC} -i ${TEMPDIR}/RGBcolorref_allsources.cat_$$ -t PSSC \
    -c "(REF_MAG_B-REF_MAG_V);" -n REF_B-V "" -k FLOAT \
    -c "(REF_MAG_V-REF_MAG_R);" -n REF_V-R "" -k FLOAT \
    -o ${CALIBDIR}/RGBcolorref_allsources.cat

# make a skycat catalogue
${P_LDACFILTER} -i ${CALIBDIR}/RGBcolorref_allsources.cat -t PSSC \
    -c "((((REF_B-V>${BVMIN})AND(REF_B-V<${BVMAX}))AND(REF_V-R<${VRMAX}))AND(REF_V-R>${VRMIN}));" \
    -o ${TEMPDIR}/colorrefforskycat_$$

${P_LDACTOSKYCAT} -i ${TEMPDIR}/colorrefforskycat_$$ -t PSSC \
    -k SeqNr Ra_G Dec_G REF_MAG_V \
    -l id_col SeqNr ra_col Ra_G dec_col \
    Dec_G mag_col REF_MAG_V \
    -s > /${CALIBDIR}/PHOTCAT_sources.skycat

# if no sources are matched ...
nobj=`wc ${TEMPDIR}/colcorr_ascii.cat_$$ | ${P_GAWK} '{print $1}'`
if [ "${nobj}" = "0" ]; then
    echo 0           >  ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    echo 1.000 0.000 >> ${CALIBDIR}/PHOTCAT_calibration.dat
    exit
fi

# obtain the colours in units of fluxes
${P_GAWK} '{print 10^(0.4*($1-$2)), 1.0, 10^(0.4*($3-$2))}' \
    ${TEMPDIR}/colcorr_ascii.cat_$$ >> ${TEMPDIR}/colcorr_ascii_fluxes.cat_tmp1_$$

${P_SORT} -n -k 1 ${TEMPDIR}/colcorr_ascii_fluxes.cat_tmp1_$$ >\
    ${TEMPDIR}/colcorr_ascii_fluxes.cat_$$

# calculate the averages
${P_GAWK} 'BEGIN {b=0;r=0} {b=b+$1; r=r+$3; barr[NR]=$1; rarr[NR]=$3} 
       END {meanb=b/NR
            meanr=r/NR
            rmsb=0
            rmsr=0
            for (i=1;i<=NR;i++) {
                rmsb=rmsb+(barr[i]-meanb)^2
                rmsr=rmsr+(rarr[i]-meanr)^2
            }
            if (NR>1) {
              rmsb=sqrt(1/(NR-1)*rmsb)/sqrt(NR)
              rmsr=sqrt(1/(NR-1)*rmsr)/sqrt(NR)
            }
            if (NR==1) {
              rmsb=0.0
              rmsr=0.0
            }
            print NR
            printf "%2.3f %2.3f\n%2.3f %2.3f\n%2.3f %2.3f\n", meanb, rmsb, 1.0, 0.0, meanr, rmsr}' \
       ${TEMPDIR}/colcorr_ascii_fluxes.cat_$$ >> \
       ${CALIBDIR}/PHOTCAT_calibration.dat
