#!/bin/bash -xv

# script that creates an illumination correction and
# fringe image out of a blank sky field (superflat)

# $1: main dir (filter)
# $2: Science directory
# $3: chips to be processed

# preliminary work:
INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

SCRIPTDIR=`pwd`

for CHIP in $3
do
  NUMTHREAD=`${SCRIPTDIR}/get_nthreads.sh ${NCHIPS} ${NPARA} ${CHIP}`

  FILE=`ls /$1/$2/$2_${CHIP}.fits`

  if [ -L ${FILE} ]; then
      LINK=`${P_READLINK} ${FILE}`
      BASE=`basename ${LINK} .fits`
      DIR=`dirname ${LINK}`
      ln -s ${DIR}/$2_${CHIP}_illum.fits $1/$2/$2_${CHIP}_illum.fits
      ln -s ${DIR}/$2_${CHIP}_fringe.fits $1/$2/$2_${CHIP}_fringe.fits
      RESULTDIR[${CHIP}]=`dirname ${LINK}`    
  else
      RESULTDIR[${CHIP}]="$1/$2"
  fi

  ${P_SEX} /$1/$2/$2_${CHIP}.fits -c ${CONF}/illumfringe_back.sex \
      -CHECKIMAGE_NAME ${RESULTDIR[${CHIP}]}/$2_${CHIP}_illum.fits \
      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
  ${P_SEX} /$1/$2/$2_${CHIP}.fits -c ${CONF}/illumfringe_fringe.sex \
      -CHECKIMAGE_NAME ${RESULTDIR[${CHIP}]}/$2_${CHIP}_fringe.fits \
      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
  if [ ! "${V_SMOOTHFRINGEBACKSIZE}_A" = "_A" ]; then
      ${P_FITSMEDIAN} -i ${RESULTDIR[${CHIP}]}/$2_${CHIP}_fringe.fits \
	  -o ${RESULTDIR[${CHIP}]}/$2_${CHIP}_fringe_s.fits \
	  -s ${V_SMOOTHFRINGEBACKSIZE}
      mv ${RESULTDIR[${CHIP}]}/$2_${CHIP}_fringe_s.fits \
	  ${RESULTDIR[${CHIP}]}/$2_${CHIP}_fringe.fits
  fi
done
