#!/bin/bash -xv

#$1: main directory (filter)
#$2: Science directory
#$3: offtarget directory
#$4: mode1 | mode2
#$5: chips to be processed

# preliminary work:

SCRIPTDIR=`pwd`

INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

# the resultdir is where the output coadded images
# will go. If ONE image of the corresponding chip
# is a link the image will go to THIS directory
for CHIP in $5
do
  RESULTDIR_SCI[${CHIP}]="$1/$2"
  RESULTDIR_OFF[${CHIP}]="$1/$3"
done

# subtract objects

if [ "${V_CAL_WINDOWMETHOD}" = "0" ]; then
    WINDOWMETHOD="superflat"
fi
if [ "${V_CAL_WINDOWMETHOD}" = "1" ]; then
    WINDOWMETHOD="defringe"
fi
if [ "${V_CAL_WINDOWMETHOD}" = "2" ]; then
    WINDOWMETHOD="subtract"
fi

. bash.include

SUB=""
# create object-subtracted images first, for OFFTARGET only

if [ "${V_CAL_DETECTMINAREA}" != "" ] || [ "${V_CAL_DETECTTHRESH}" != "" ]; then
    SUB=".mask"
    for CHIP in $5
    do
      NUMTHREAD=`${SCRIPTDIR}/get_nthreads.sh ${NCHIPS} ${NPARA} ${CHIP}`

      ls -1 /${RESULTDIR_OFF[${CHIP}]}/*_${CHIP}OFC.fits \
	  > ${TEMPDIR}/images-objects_$$
	
      cat ${TEMPDIR}/images-objects_$$ |\
      {
        while read file
	do
	  BASE=`basename ${file} .fits`
	  # if there are already masks in MASK_IMAGES we don't need to run SExtractor

	  if [ -f /$1/$3/MASK_IMAGES/${BASE}U.mask.fits ]; then
	      ${P_IC} '%1 -1.0e09 %2 -5.0e08 > ?' ${file} /$1/$3/MASK_IMAGES/${BASE}U.mask.fits \
		  > ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}.fits"
	  fi

	  # if there are no masks, we need to run SExtractor to subtract objects from the image
	  if [ ! -f /$1/$3/MASK_IMAGES/${BASE}U.mask.fits ]; then
	      ${P_SEX} ${file} -c ${DATACONF}/image-objects.sex\
		  -CHECKIMAGE_NAME ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}.fits" \
		  -DETECT_MINAREA ${V_CAL_DETECTMINAREA} \
		  -DETECT_THRESH ${V_CAL_DETECTTHRESH} \
		  -ANALYSIS_THRESH ${V_CAL_DETECTTHRESH} \
		  -BACK_SIZE 256 \
		  -FILTER ${V_CAL_SEXFILTER} \
		  -NTHREADS ${NUMTHREAD}
	      
	      ${P_IC} '%1 -1.0e09 %2 fabs 1.0e-06 > ?' ${file} \
		  ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}.fits" \
		  > ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}1.fits"
	      
	      mv ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}1.fits" \
		  ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}.fits"	  
	  fi
	  
	  if [ "${RESULTDIR_OFF[${CHIP}]}" != "$1/$3" ]; then
	      ln -s ${RESULTDIR_OFF[${CHIP}]}/${BASE}"${SUB}.fits" \
		  $1/$3/${BASE}"${SUB}.fits"
	  fi
	done
      }
    done
fi

# dynamic superflat

#################################################################################
# correction of the SCIENCE images
#################################################################################

for CHIP in $5
do
  if [ "${RESULTDIR_OFF[${CHIP}]}" != "$1/$3" ]; then
      ln -s ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}.fits $1/$3/$3_${CHIP}.fits
  fi
  
  NUMTHREAD=`${SCRIPTDIR}/get_nthreads.sh ${NCHIPS} ${NPARA} ${CHIP}`

  ls -1 /$1/$2/*_${CHIP}OFC.fits > ${TEMPDIR}/images_to_flat_$$
  
  k=1

  # loop over all images
  cat ${TEMPDIR}/images_to_flat_$$ |\
  {
    while read individualfile
    do
      # get the image names from which to calculate the superflat
      num=`${P_FIND} /$1/$3/ -maxdepth 1 -name \*_${CHIP}OFC${SUB}.fits | wc | ${P_GAWK} '{print $1}'`
#      num=`ls -1 /$1/$3/*_${CHIP}OFC${SUB}.fits | wc | ${P_GAWK} '{print $1}'`
      win=${V_CAL_WINDOWSIZE}
      # include the n=window images before and after the exposure

      # the MJD of the exposure that should be corrected
      dateobs=`${P_DFITS} ${individualfile} | ${P_FITSORT} DATE-OBS | ${P_GAWK} '(NR>1) {print $2}'`
      mjd_science=`${P_MJD} -f ${dateobs} | ${P_GAWK} '{print $7}'`

      # the names and MJDs of the offtarget exposures
      ls /$1/$3/*_${CHIP}OFC.fits > ${TEMPDIR}/offtarget_mjds_list_$$

      test -f ${TEMPDIR}/offtarget_mjds_$$ && rm ${TEMPDIR}/offtarget_mjds_$$

      cat ${TEMPDIR}/offtarget_mjds_list_$$ |\
      {
	while read file
        do
	  dateobs=`${P_DFITS} ${file} | ${P_FITSORT} DATE-OBS | \
	      ${P_GAWK} '(NR>1) {print $2}'`
	  mjd=`${P_MJD} -f ${dateobs} | ${P_GAWK} '{print $7}'`
	  echo ${file} ${mjd} >> ${TEMPDIR}/offtarget_mjds_$$
        done
      }
      
      # select the n=window images before and after the science MJD
      images=`${P_GAWK} 'BEGIN {
              l='"${win}"'
              n='"${num}"'
	      mjdsci='"${mjd_science}"'} {
	      name[NR]=$1
	      mjdoff[NR]=$2} END {
	      position=1 
	      for(j=1;j<=NR;j++) {
		  if (mjdoff[j] < mjdsci) position++
	      }
	      start=position-l
	      if (position-l<1) start=1
	      if (position+l>n) start=n-2*l+1
	      for (j=1;j<=NR;j++) { 
		  if (j>=start && j<start+2*l) print name[j]
	      }
	      }' ${TEMPDIR}/offtarget_mjds_$$`

      # background statistics
      ${P_IMSTATS} -d 6 ${images} -o ${TEMPDIR}/offtarget_images_$$ -t -1e8 1e8
      
      RESULTMODE=`${P_GAWK} 'BEGIN {mean=0.0; n=0} ($1!="#") {
              n=n+1; mean=mean+$2} END {print mean/n}' ${TEMPDIR}/offtarget_images_$$`
      
      # modify the input list of images
      # in case we have to reject files for the superflat:
      if [ -s /$1/$2/superflat_exclusion ]; then
	  ${P_GAWK} 'BEGIN {nex = 0; while (getline <"/'$1/$2'/superflat_exclusion" > 0) {
                   ex[nex]=$1; nex++; }}
                   {exclude = 0; i=0;
                    while (i<nex) {
                      if (index($1, ex[i])!=0) {
                        exclude = 1;
                      }
	              i++;
                    }
                    if(exclude == 0) {print $0}}' ${TEMPDIR}/offtarget_images_$$ >\
			${TEMPDIR}/offtarget_coadd_images_$$
      else
	  cp ${TEMPDIR}/offtarget_images_$$ ${TEMPDIR}/offtarget_coadd_images_$$
      fi
      
      # do the combination; do not include images
      # listed in the /$1/$2/superflat_exclusion file
	  
      ${P_IMCOMBFLAT_IMCAT} -i ${TEMPDIR}/offtarget_coadd_images_$$ \
	  -o ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}.fits -c ${V_CAL_COMBINEMETHOD} \
	  -s 1 -e ${V_CAL_SCIENCENLOW} ${V_CAL_SCIENCENHIGH} -m ${RESULTMODE}  -t -1e8 1e8
      
      get_statminsection

      # now apply the superflat (superflat, defringe, or subtract)
      if [ "${WINDOWMETHOD}" != "subtract" ]; then
	  FILE=`ls /$1/$3/$3_${CHIP}.fits`
	  if [ -L ${FILE} ]; then
	      LINK=`${P_READLINK} ${FILE}`
	      BASE=`basename ${LINK} .fits`
	      DIR=`dirname ${LINK}`
	      ln -s ${DIR}/$3_${CHIP}_illum.fits $1/$3/$3_${CHIP}_illum.fits
	      ln -s ${DIR}/$3_${CHIP}_fringe.fits $1/$3/$3_${CHIP}_fringe.fits
	  fi
	  
	  ${P_SEX} /$1/$3/$3_${CHIP}.fits -c ${CONF}/illumfringe_back.sex \
	      -CHECKIMAGE_NAME ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}_illum.fits \
	      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
	  ${P_SEX} /$1/$3/$3_${CHIP}.fits -c ${CONF}/illumfringe_fringe.sex \
	      -CHECKIMAGE_NAME ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}_fringe.fits \
	      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
	  
	  # only superflat the data
	  if [ "${WINDOWMETHOD}" = "superflat" ]; then
	      if [ "${V_SF_SMOOTH}" = "SMOOTHED" ]; then
		  FLATIMAGE=/$1/$3/$3_${CHIP}_illum.fits
	      fi
	      if [ "${V_SF_SMOOTH}" = "UNSMOOTHED" ]; then
		  FLATIMAGE=/$1/$3/$3_${CHIP}.fits
	      fi
	      
	      ${P_IMRED_ECL:?} ${individualfile} \
		  -c ${CONF}/imred.conf \
		  -MAXIMAGES ${NFRAMES} \
		  -OVERSCAN N \
		  -BIAS N \
		  -FLAT Y \
		  -FLAT_IMAGE ${FLATIMAGE} \
		  -FLAT_THRESHHOLD 0.1 \
		  -FLAT_SCALE N \
		  -COMBINE N \
		  -OUTPUT Y \
		  -OUTPUT_DIR /$1/$2/ \
		  -OUTPUT_SUFFIX S.fits \
		  -TRIM N \
		  -STATSSEC ${statminsection}
	  fi
	  
	  # superflat and defringe the data
	  if [ "${WINDOWMETHOD}" = "defringe" ]; then
	      FLATIMAGE=/$1/$3/$3_${CHIP}_illum.fits
	      FRINGEIMAGE=/$1/$3/$3_${CHIP}_fringe.fits
	      ${P_IMRED_ECL:?} ${individualfile} \
		  -c ${CONF}/imred.conf \
		  -MAXIMAGES ${NFRAMES} \
		  -OVERSCAN N \
		  -BIAS N \
		  -FLAT Y \
		  -FLAT_IMAGE ${FLATIMAGE} \
		  -FLAT_THRESHHOLD 0.1 \
		  -FLAT_SCALE N \
		  -FRINGE Y \
		  -FRINGE_IMAGE ${FRINGEIMAGE} \
		  -FRINGE_REFIMAGE ${FLATIMAGE} \
		  -FRINGE_SCALE ${V_FRINGESCALE} \
		  -COMBINE N \
		  -OUTPUT Y \
		  -OUTPUT_DIR /$1/$2/ \
		  -OUTPUT_SUFFIX SF.fits \
		  -TRIM N \
		  -STATSSEC ${statminsection}
	  fi
      fi
      
      # subtract superflat
      if [ "${WINDOWMETHOD}" = "subtract" ]; then
	  FRINGEIMAGE=/$1/$3/$3_${CHIP}.fits
	  ${P_IMRED_ECL:?} ${individualfile} \
	      -c ${CONF}/imred.conf \
	      -MAXIMAGES ${NFRAMES} \
	      -OVERSCAN N \
	      -BIAS N \
	      -FLAT N \
	      -FRINGE Y \
	      -FRINGE_IMAGE ${FRINGEIMAGE} \
	      -FRINGE_REFIMAGE ${FRINGEIMAGE} \
	      -FRINGE_SCALE ${V_FRINGESCALE} \
	      -COMBINE N \
	      -OUTPUT Y \
	      -OUTPUT_DIR /$1/$2/ \
	      -OUTPUT_SUFFIX U.fits \
	      -TRIM N \
	      -STATSSEC ${statminsection}
	  
	  BASE=`basename ${individualfile} U.fits`
	  if [ "${RESULTDIR_SCI[${CHIP}]}" != "$1/$2" ]; then
	      ln -s ${RESULTDIR_SCI[${CHIP}]}/${BASE}U.fits \
		  /$1/$2/${BASE}U.fits
	  fi
      fi
      
      if [ ! -d /$1/$2/OFC_IMAGES ]; then
	  mkdir /$1/$2/OFC_IMAGES
      fi
      mv ${individualfile} /$1/$2/OFC_IMAGES

      k=$((${k}+1))
    done
  }
  
  if [ "${SUB}" = ".mask" ] && [ ! "${4}" = "mode2" ]; then
      if [ ! -d /$1/$3/MASK_IMAGES ]; then
	  mkdir /$1/$3/MASK_IMAGES
      fi
      mv /$1/$3/*_${CHIP}OFC${SUB}.fits /$1/$3/MASK_IMAGES/
  fi
done



#################################################################################################
# correct the OFFTARGET images
#################################################################################################

if [ "${4}" = "mode2" ]; then

for CHIP in $5
do
  if [ "${RESULTDIR_OFF[${CHIP}]}" != "$1/$3" ]; then
      ln -s ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}.fits $1/$3/$3_${CHIP}.fits
  fi
  
  NUMTHREAD=`${SCRIPTDIR}/get_nthreads.sh ${NCHIPS} ${NPARA} ${CHIP}`

  ls -1 /$1/$3/*_${CHIP}OFC.fits > ${TEMPDIR}/offtargetimages_to_flat_$$
  
  k=1

  # loop over all images
  cat ${TEMPDIR}/offtargetimages_to_flat_$$ |\
  {
    while read individualofftarget
    do
      # get the image names from which to calculate the superflat
      num=`ls -1 /$1/$3/*_${CHIP}OFC${SUB}.fits | wc | ${P_GAWK} '{print $1}'`
      win=${V_CAL_WINDOWSIZE}
      # include the n=window images before and after the exposure

      # the MJD of the exposure that should be corrected
      dateobs=`${P_DFITS} ${individualofftarget} | ${P_FITSORT} DATE-OBS | ${P_GAWK} '(NR>1) {print $2}'`
      mjd_science=`${P_MJD} -f ${dateobs} | ${P_GAWK} '{print $7}'`

      # the names and MJDs of the offtarget exposures
      ls /$1/$3/*_${CHIP}OFC.fits > ${TEMPDIR}/offtarget_mjds_list_$$

      test -f ${TEMPDIR}/offtarget_mjds_$$ && rm ${TEMPDIR}/offtarget_mjds_$$

      cat ${TEMPDIR}/offtarget_mjds_list_$$ |\
      {
	while read file
        do
	  dateobs=`${P_DFITS} ${file} | ${P_FITSORT} DATE-OBS | \
	      ${P_GAWK} '(NR>1) {print $2}'`
	  mjd=`${P_MJD} -f ${dateobs} | ${P_GAWK} '{print $7}'`
	  echo ${file} ${mjd} >> ${TEMPDIR}/offtarget_mjds_$$
        done
      }
      
      # get the image names from which to calculate the superflat
      num=`ls -1 /$1/$3/*_${CHIP}OFC${SUB}.fits | wc | ${P_GAWK} '{print $1}'`
      win=${V_CAL_WINDOWSIZE}
      ind=${k}
      offimages=`ls -1 /$1/$3/*_${CHIP}OFC${SUB}.fits | ${P_GAWK} 'BEGIN {
                l='"${win}"'
                i='"${ind}"'
                n='"${num}"'
                j=1
                start=i-l
                test=0
                if (i-l<1) {
                   start=1
                   test=i-l-1
                }
                if (i+l>n) {
                   start=n-2*l
                   test=i+l-n
                }
	      } {
              if (NR>=start && j<=2*l+1) {
                 if (j != l+1+test) print
                 j++
              } }'`

      # background statistics
      ${P_IMSTATS} -d 6 ${offimages} -o ${TEMPDIR}/offtarget_offimages_$$ -t -1e8 1e8
      
      RESULTMODE=`${P_GAWK} 'BEGIN {mean=0.0; n=0} ($1!="#") {
		       n=n+1; mean=mean+$2} END {print mean/n}' ${TEMPDIR}/offtarget_offimages_$$`
      
      # modify the input list of images
      # in case we have to reject files for the superflat:
      if [ -s /$1/$2/superflat_exclusion ]; then
	  ${P_GAWK} 'BEGIN {nex = 0; while (getline <"/'$1/$2'/superflat_exclusion" > 0) {
                   ex[nex]=$1; nex++; }}
                   {exclude = 0; i=0;
                    while (i<nex) {
                      if (index($1, ex[i])!=0) {
                        exclude = 1;
                      }
	              i++;
                    }
                    if(exclude == 0) {print $0}}' ${TEMPDIR}/offtarget_offimages_$$ >\
			${TEMPDIR}/offtarget_offcoadd_images_$$
      else
	  cp ${TEMPDIR}/offtarget_offimages_$$ ${TEMPDIR}/offtarget_offcoadd_images_$$
      fi
      
      # do the combination; do not include images
      # listed in the /$1/$2/superflat_exclusion file
      
      ${P_IMCOMBFLAT_IMCAT} -i ${TEMPDIR}/offtarget_offcoadd_images_$$ \
	  -o ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}.fits -c ${V_CAL_COMBINEMETHOD} \
	  -s 1 -e ${V_CAL_SCIENCENLOW} ${V_CAL_SCIENCENHIGH} -m ${RESULTMODE}  -t -1e8 1e8
      
      # the statistics area
      get_statminsection

      # now apply the superflat (superflat, defringe, or subtract)
      if [ "${WINDOWMETHOD}" != "subtract" ]; then
	  FILE=`ls /$1/$3/$3_${CHIP}.fits`
	  if [ -L ${FILE} ]; then
	      LINK=`${P_READLINK} ${FILE}`
	      BASE=`basename ${LINK} .fits`
	      DIR=`dirname ${LINK}`
	      ln -s ${DIR}/$3_${CHIP}_illum.fits $1/$3/$3_${CHIP}_illum.fits
	      ln -s ${DIR}/$3_${CHIP}_fringe.fits $1/$3/$3_${CHIP}_fringe.fits
	  fi
	  
	  ${P_SEX} /$1/$3/$3_${CHIP}.fits -c ${CONF}/illumfringe_back.sex \
	      -CHECKIMAGE_NAME ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}_illum.fits \
	      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
	  ${P_SEX} /$1/$3/$3_${CHIP}.fits -c ${CONF}/illumfringe_fringe.sex \
	      -CHECKIMAGE_NAME ${RESULTDIR_OFF[${CHIP}]}/$3_${CHIP}_fringe.fits \
	      -BACK_SIZE ${V_ILLUMFRINGEBACKSIZE} -NTHREADS ${NUMTHREAD}
	  
	  # only superflat the data
	  if [ "${WINDOWMETHOD}" = "superflat" ]; then
	      if [ "${V_SF_SMOOTH}" = "SMOOTHED" ]; then
		  FLATIMAGE=/$1/$3/$3_${CHIP}_illum.fits
	      fi
	      if [ "${V_SF_SMOOTH}" = "UNSMOOTHED" ]; then
		  FLATIMAGE=/$1/$3/$3_${CHIP}.fits
	      fi
	      
	      ${P_IMRED_ECL:?} ${individualofftarget} \
		  -c ${CONF}/imred.conf \
		  -MAXIMAGES ${NFRAMES} \
		  -OVERSCAN N \
		  -BIAS N \
		  -FLAT Y \
		  -FLAT_IMAGE ${FLATIMAGE} \
		  -FLAT_THRESHHOLD 0.1 \
		  -FLAT_SCALE N \
		  -COMBINE N \
		  -OUTPUT Y \
		  -OUTPUT_DIR /$1/$3/ \
		  -OUTPUT_SUFFIX S.fits \
		  -TRIM N \
		  -STATSSEC ${statminsection}
	  fi
	  
	  # superflat and defringe the data
	  if [ "${WINDOWMETHOD}" = "defringe" ]; then
	      FLATIMAGE=/$1/$3/$3_${CHIP}_illum.fits
	      FRINGEIMAGE=/$1/$3/$3_${CHIP}_fringe.fits
	      ${P_IMRED_ECL:?} ${individualofftarget} \
		  -c ${CONF}/imred.conf \
		  -MAXIMAGES ${NFRAMES} \
		  -OVERSCAN N \
		  -BIAS N \
		  -FLAT Y \
		  -FLAT_IMAGE ${FLATIMAGE} \
		  -FLAT_THRESHHOLD 0.1 \
		  -FLAT_SCALE N \
		  -FRINGE Y \
		  -FRINGE_IMAGE ${FRINGEIMAGE} \
		  -FRINGE_REFIMAGE ${FLATIMAGE} \
		  -FRINGE_SCALE ${V_FRINGESCALE} \
		  -COMBINE N \
		  -OUTPUT Y \
		  -OUTPUT_DIR /$1/$3/ \
		  -OUTPUT_SUFFIX SF.fits \
		  -TRIM N \
		  -STATSSEC ${statminsection}
	  fi
      fi
      
      # subtract superflat
      if [ "${WINDOWMETHOD}" = "subtract" ]; then
	  FRINGEIMAGE=/$1/$3/$3_${CHIP}.fits
	  ${P_IMRED_ECL:?} ${individualofftarget} \
	      -c ${CONF}/imred.conf \
	      -MAXIMAGES ${NFRAMES} \
	      -OVERSCAN N \
	      -BIAS N \
	      -FLAT N \
	      -FRINGE Y \
	      -FRINGE_IMAGE ${FRINGEIMAGE} \
	      -FRINGE_REFIMAGE ${FRINGEIMAGE} \
	      -FRINGE_SCALE ${V_FRINGESCALE} \
	      -COMBINE N \
	      -OUTPUT Y \
	      -OUTPUT_DIR /$1/$3/ \
	      -OUTPUT_SUFFIX U.fits \
	      -TRIM N \
	      -STATSSEC ${statminsection}
	  
	  BASE=`basename ${individualofftarget} U.fits`
	  if [ "${RESULTDIR_OFF[${CHIP}]}" != "$1/$3" ]; then
	      ln -s ${RESULTDIR_OFF[${CHIP}]}/${BASE}U.fits \
		  /$1/$3/${BASE}U.fits
	  fi
      fi
      
      if [ ! -d /$1/$3/OFC_IMAGES ]; then
	  mkdir /$1/$3/OFC_IMAGES
      fi
      mv ${individualofftarget} /$1/$3/OFC_IMAGES
      
      k=$((${k}+1))
      
    done
  }
  
  if [ "${SUB}" = ".mask" ]; then
      if [ ! -d /$1/$3/MASK_IMAGES ]; then
	  mkdir /$1/$3/MASK_IMAGES
      fi
      mv /$1/$3/*_${CHIP}OFC${SUB}.fits /$1/$3/MASK_IMAGES/
  fi
  
done

fi
