#!/bin/bash -xv

# the script creates sky subtracted images.
# It puts them in the same dir. as the science frames

# $1: main directory
# $2: science directory
# $3: extension
# $4: cut extension
# $5: new extension
# $6: from which chip
# $7: chips to be processed

SCRIPTDIR=`pwd`

INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

. bash.include

region=""
if [ "${V_CSKY_RA1}" != "" ]  && [ "${V_CSKY_RA2}" != "" ] && \
   [ "${V_CSKY_DEC1}" != "" ] && [ "${V_CSKY_DEC2}" != "" ]; then
    region="from_region"
fi

section=""
if [ "${V_CSKY_XMIN}" != "" ] && [ "${V_CSKY_XMAX}" != "" ] && \
   [ "${V_CSKY_YMIN}" != "" ] && [ "${V_CSKY_YMAX}" != "" ]; then

    section="-s ${V_CSKY_XMIN} ${V_CSKY_XMAX} ${V_CSKY_YMIN} ${V_CSKY_YMAX}"
fi

# the chips to process

for CHIP in $7
do
  ls -1 /$1/$2/*_${CHIP}$3$4fits > ${TEMPDIR}/skysub_images${CHIP}_$$

  FILE=`${P_GAWK} '(NR==1) {print $0}' ${TEMPDIR}/skysub_images${CHIP}_$$`

  if [ -L ${FILE} ]; then
      LINK=`${P_READLINK} ${FILE}`
      RESULTDIR[${CHIP}]=`dirname ${LINK}`
  else
      RESULTDIR[${CHIP}]="$1/$2"
  fi  
  
  NUMTHREAD=`${SCRIPTDIR}/get_nthreads.sh ${NCHIPS} ${NPARA} ${CHIP}`

  cat ${TEMPDIR}/skysub_images${CHIP}_$$ |\
  {
  while read file
    do
      BASE=`basename ${file} $4fits`
      BASEred=`basename ${file} ${CHIP}$3$4fits`

      # get the statistics required
      # first, the method (mode - median - mean - lowquart)
      if [ "${V_CSKYMETHOD}" = "0" ]; then
	  pos="2"
      fi
      if [ "${V_CSKYMETHOD}" = "1" ]; then
	  pos="4"
      fi
      if [ "${V_CSKYMETHOD}" = "2" ]; then
	  pos="6"
      fi
      if [ "${V_CSKYMETHOD}" = "3" ]; then
	  pos="3"
      fi

      # get the corresponding sky value

      # do it for each chip
      if [ "$6" = "ALL" ] && [ "${region}" != "from_region" ]; then
          # first run sextractor to subtract objects from the image
	  ${P_SEX} ${file} -c ${DATACONF}/image-objects.sex \
	      -DETECT_THRESH ${V_SKYSUBDETECTTHRESH} \
	      -DETECT_MINAREA ${V_SKYSUBDETECTMINAREA} \
	      -CHECKIMAGE_NAME ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits" \
	      -NTHREADS ${NUMTHREAD}
      
	  ${P_IC} '%1 -1.0e09 %2 fabs 1.0e-06 > ?' ${file} \
	      ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits" \
	      > ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"
	  \rm ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits"
	  
	  if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
	      ln -s ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits" \
		  /$1/$2/${BASE}"_skysub.fits"
	  fi

	  sky=`${P_IMSTATS} -d 6 ${section} -t -1e8 1e8 $1/$2/${BASE}"_skysub.fits" |\
               ${P_GAWK} '($1 != "#") {print $'"${pos}"'}'`

	  \rm $1/$2/${BASE}"_skysub.fits"

	  if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
	      \rm ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"
	  fi
      fi

      # get the sky from one chip (mosaic cameras only)
      if [ "$6" != "ALL" ] && \
	 [ "${V_CSKYMANUAL}" != "MANUAL" ] && [ "${region}" != "from_region" ]; then

          # run sextractor to subtract objects from the image
	  # in principle, this step has to be done only once, but it
          # cannot be parallelised. thus, each processor repeats it

	  ${P_SEX} $1/$2/${BASEred}$6$3$4fits -c ${DATACONF}/image-objects.sex \
	      -DETECT_THRESH ${V_SKYSUBDETECTTHRESH} \
	      -DETECT_MINAREA ${V_SKYSUBDETECTMINAREA} \
	      -CHECKIMAGE_NAME ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_sub1"$4"fits"_$$ \
	      -NTHREADS ${NUMTHREAD}

	  ${P_IC} '%1 -1.0e09 %2 fabs 1.0e-06 > ?' $1/$2/${BASEred}$6$3$4fits \
	      ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_sub1"$4"fits"_$$ \
	      > ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_skysub"$4"fits"_$$
	  \rm ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_sub1"$4"fits"_$$
	  
	  if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
	      ln -s ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_skysub"$4"fits"_$$ \
		  /$1/$2/${BASEred}$6$3"_skysub"$4"fits"_$$
	  fi
	  
	  sky=`${P_IMSTATS} -d 6 ${section} -t -1e8 1e8 $1/$2/${BASEred}$6$3"_skysub"$4"fits"_$$ |\
                   ${P_GAWK} '($1 != "#") {print $'"${pos}"'}'`
      fi

      # get the sky from a region (all cameras)
      if [ "${region}" = "from_region" ] && \
	 [ "${V_CSKYMANUAL}" != "MANUAL" ]; then

	  # identify in which chip the region is located
	  ls -1 /$1/$2/${BASEred}*$3$4"fits" > ${TEMPDIR}/sky2xy_imagelist_$$
	  ${P_SKY2XY_HELPER} -i ${TEMPDIR}/sky2xy_imagelist_$$ \
	      -c ${V_CSKY_RA1} ${V_CSKY_DEC1} ${V_CSKY_RA2} ${V_CSKY_DEC2} \
	      > ${TEMPDIR}/sky2xy_result_$$

	  image=`${P_GAWK} '{print $1}' ${TEMPDIR}/sky2xy_result_$$`
	  section=`${P_GAWK} '{print $2, $3, $4, $5}' ${TEMPDIR}/sky2xy_result_$$`

          # run sextractor to subtract objects from the image
	  # in principle, this step has to be done only once, but it
          # cannot be parallelised. thus, each processor repeats it

	  ${P_SEX} ${image} -c ${DATACONF}/image-objects.sex \
	      -DETECT_THRESH ${V_SKYSUBDETECTTHRESH} \
	      -DETECT_MINAREA ${V_SKYSUBDETECTMINAREA} \
	      -CHECKIMAGE_NAME ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits"_$$ \
	      -NTHREADS ${NUMTHREAD}

	  ${P_IC} '%1 -1.0e09 %2 fabs 1.0e-06 > ?' ${image} \
	      ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits"_$$ \
	      > ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"_$$
	  \rm ${RESULTDIR[${CHIP}]}/${BASE}"_sub1.fits"_$$
	  
	  if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
	      ln -s ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"_$$ \
		  /$1/$2/${BASE}"_skysub.fits"_$$
	  fi
	  
	  sky=`${P_IMSTATS} -d 6 $1/$2/${BASE}"_skysub.fits"_$$ \
               -t -1e8 1e8 -s ${section} | ${P_GAWK} '($1 != "#") {print $'"${pos}"'}'`	      
      fi

      # finally subtract the sky
      if [ "${V_CSKYMANUAL}" != "MANUAL" ]; then
	  ${P_IC} '%1 '"${sky}"' -' ${file} > ${RESULTDIR[${CHIP}]}/${BASE}"$5fits"
	  test -f ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_skysub"$4"fits"_$$ \
	      && \rm ${RESULTDIR[${CHIP}]}/${BASEred}$6$3"_skysub"$4"fits"_$$
	  test -f ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"_$$ \
	      && \rm ${RESULTDIR[${CHIP}]}/${BASE}"_skysub.fits"_$$
      fi

     # what to do if a manual sky has been defined
      if [ "${V_CSKYMANUAL}" = "MANUAL" ]; then
	  sky=`grep ${BASE} $1/$2/skybackground.use | ${P_GAWK} '{print $2}'`
	  ${P_IC} '%1 '"${sky}"' -' ${file} > ${RESULTDIR[${CHIP}]}/${BASE}"$5fits"
      fi

      # write the sky into the fits header
      skyexist=`get_key ${RESULTDIR[${CHIP}]}/${BASE}"$5fits" SKYVALUE`
      if [ "${skyexist}"_A = "_A" ]; then
	  NEXTDUMMY=`${P_DFITS} ${RESULTDIR[${CHIP}]}/${BASE}"$5fits" | grep DUMMY | \
	      ${P_GAWK} 'BEGIN {nrmin=1000+0} {
                        if (index($1,"DUMMY")==1) {
			   nr=substr($1,6)+0; if(nr<nrmin) {nrmin=nr}}} 
                        END {if(NR>0) {print nrmin}}'`
	  ${P_REPLACEKEY} ${RESULTDIR[${CHIP}]}/${BASE}"$5fits" \
	      "SKYVALUE= ${sky}" DUMMY${NEXTDUMMY}
      fi

      if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
	  ln -s ${RESULTDIR[${CHIP}]}/${BASE}"$5fits" /$1/$2/${BASE}"$5fits"
      fi
    done
  }
done
