#!/bin/bash -xv

# the script corrects a set of Science frames
# for illumination pattern. This is an effective
# second flat-fielding. 
# The script uses the 'preprocess' program to perform the
# reduction.
# It assumes that all images from ONE chip reside
# in the same physical directory
#
# 27.10.2004
# added the possbility to use the unsmoothed superflat instead of
# the smoothed superflat for the illumination correction
#

# $1: main directory (filter)
# $2: Science directory
# $3: chips to be processed

# preliminary work:
INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

. bash.include

for CHIP in $3
do
  RESULTDIR[${CHIP}]="$1/$2"
done

for CHIP in $3
do

  FILES=`ls /$1/$2/*_${CHIP}OFC.fits`

  for FILE in ${FILES}
  do
    if [ -L ${FILE} ]; then
	 LINK=`${P_READLINK} ${FILE}`
	 BASE=`basename ${LINK} .fits`
	 DIR=`dirname ${LINK}`
	 ln -s ${DIR}/${BASE}S.fits $1/$2/${BASE}S.fits
	 RESULTDIR[${CHIP}]=`dirname ${LINK}`    
    fi
  done 

  if [ "${V_SF_SMOOTH}" = "SMOOTHED" ]; then
      FLATIMAGE=/$1/$2/$2_${CHIP}_illum.fits
  fi
  if [ "${V_SF_SMOOTH}" = "UNSMOOTHED" ]; then
      FLATIMAGE=/$1/$2/$2_${CHIP}.fits
  fi

  get_statminsection

  ${P_IMRED_ECL:?} ${FILES} \
      -c ${CONF}/imred.conf \
      -MAXIMAGES ${NFRAMES} \
      -OVERSCAN N \
      -BIAS N \
      -FLAT Y \
      -FLAT_IMAGE ${FLATIMAGE} \
      -FLAT_SCALE ${V_ADJUSTGAINS} \
      -FLAT_SCALEIMAGE ${FLATIMAGE} \
      -FLAT_THRESHHOLD 0.1 \
      -COMBINE N \
      -OUTPUT Y \
      -OUTPUT_DIR /$1/$2/ \
      -OUTPUT_SUFFIX S.fits \
      -TRIM N \
      -STATSSEC ${statminsection}

  if [ ! -d /$1/$2/OFC_IMAGES ]; then
     mkdir /$1/$2/OFC_IMAGES
  fi 
  mv /$1/$2/*_${CHIP}OFC.fits /$1/$2/OFC_IMAGES

  i=$(( $i + 1 ))
done
