#!/bin/bash -xv

# the script corrects a set of Science frames
# for fringe pattern. 
# The script uses the 'preprocess' program to perform the
# reduction.
# It assumes that all images from ONE chip reside
# in the same physical directory

# It subtracts the unsmoothed SUPERFLAT directly from the
# OFC images!

# $1: main directory (filter)
# $2: science directory
# $3: chips to be processed

# preliminary work:
INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

. bash.include

for CHIP in $3
do
  RESULTDIR[${CHIP}]="$1/$2"
done

for CHIP in $3
do

  if [ ! -e /$1/$2/$2_${CHIP}.fits ]; then
      echo `${P_ERRTEST} 0` Could not find /$1/$2/$2_${CHIP}.fits
      exit
  fi

  FILES=`ls $1/$2/*_${CHIP}OFC.fits`

  for FILE in ${FILES}
  do
    if [ -L ${FILE} ]; then
	 LINK=`${P_READLINK} ${FILE}`
	 BASE=`basename ${LINK} .fits`
	 DIR=`dirname ${LINK}`
	 ln -s ${DIR}/${BASE}U.fits $1/$2/${BASE}U.fits
	 RESULTDIR[${CHIP}]=`dirname ${LINK}`  
    fi  
  done 

  get_statminsection

  ${P_IMRED_ECL:?} ${FILES} \
      -c ${CONF}/imred.conf \
      -MAXIMAGES ${NFRAMES} \
      -OVERSCAN N \
      -BIAS N \
      -FLAT N \
      -COMBINE N \
      -FRINGE Y \
      -FRINGE_IMAGE /$1/$2/$2_${CHIP}.fits \
      -FRINGE_REFIMAGE /$1/$2/$2_${CHIP}.fits \
      -FRINGE_SCALE ${V_FRINGESCALE} \
      -OUTPUT Y \
      -OUTPUT_DIR /$1/$2/ \
      -OUTPUT_SUFFIX U.fits \
      -TRIM N \
      -STATSSEC ${statminsection}

  for FILE in ${FILES}
  do
    # write the sky background into the header
    skyexist=`get_key ${FILE} SKYVALUE`
    if [ "${skyexist}"_A = "_A" ]; then
	NEXTDUMMY=`${P_DFITS} ${FILE} | grep DUMMY | \
	    ${P_GAWK} 'BEGIN {nrmin=1000+0} {
                if (index($1,"DUMMY")==1) {
			nr=substr($1,6)+0; if(nr<nrmin) {nrmin=nr}}}
            END {if(NR>0) {print nrmin}}'`
	sky=`${P_IMSTATS} -d 6 ${FILE} -t -1e8 1e8 | \
	    ${P_GAWK} '($1 != "#") {print $2}'`
#	sky=`${P_IMSTATS} -d 6 /$1/$2/$2_${CHIP}.fits -t -1e8 1e8 | \
#	    ${P_GAWK} '($1 != "#") {print $2}'`
	BASE=`basename ${FILE} .fits`
	${P_REPLACEKEY} ${FILE} "SKYVALUE= ${sky}" DUMMY${NEXTDUMMY}
	${P_REPLACEKEY} /$1/$2/${BASE}U.fits "SKYVALUE= ${sky}" DUMMY${NEXTDUMMY}
    fi
  done 

  FILES=`ls /$1/$2/*_${CHIP}OFCU.fits`

  for FILE in ${FILES}
  do
    BASE=`basename ${FILE} U.fits`
    if [ "${RESULTDIR[${CHIP}]}" != "$1/$2" ]; then
        ln -s ${RESULTDIR[${CHIP}]}/${BASE}U.fits \
	    /$1/$2/${BASE}U.fits
    fi
  done 

  if [ ! -d /$1/$2/OFC_IMAGES ]; then
     mkdir /$1/$2/OFC_IMAGES
  fi
  mv /$1/$2/*_${CHIP}OFC.fits /$1/$2/OFC_IMAGES

done
