#!/bin/bash -xv

# ***************************************
# ***** INSTRUMENT splitting script *****
# *****      created by THELI       *****
# ***************************************

# splits WIRCam@CFHT images

# $1: main directory
# $2: science directory

INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

. bash.include


# rename the images to the value of a FITS key (if requested)

if [ "${V_PRE_RENAME_CHECKED}" = "1" ]; then

  cd /$1/$2

  p_ls > ${TEMPDIR}/fitskey_$$

  cat ${TEMPDIR}/fitskey_$$ |\
  {
    while read FILE
    do
      newname=`get_key ${FILE} ${V_RENAME_FITSKEY}`
      if [ "${newname}" != "KEY_N/A" ] && [ "${newname}" != "" ]; then
         mv ${FILE} ${newname}
      else
         echo ERROR: Could not find ${V_RENAME_FITSKEY} in the FITS header
      fi
    done
  }
fi



if [ ! -d "/$1/$2_i" ]; then
    mkdir /$1/$2_i
fi
if [ ! -d "/$1/$2_z" ]; then
    mkdir /$1/$2_z
fi
if [ ! -d "/$1/$2_g" ]; then
    mkdir /$1/$2_g
fi
if [ ! -d "/$1/$2_r" ]; then
    mkdir /$1/$2_r
fi

FILES=`ls -1l $1/$2/ | ${P_GAWK} '($1 ~ /^-/) {print $NF}'`

cd /$1/$2

mkdir ORIGINALS

# first split the MEF file into the four filters
for FILE in ${FILES}
do
  BASE=`basename ${FILE} .fits`
  ${P_MISSFITS} ${FILE} \
      -OUTFILE_TYPE SPLIT -SPLIT_SUFFIX _sl%02d.fits -WRITE_XML N
#  mv ${FILE} ORIGINALS
done

# cut the images into the two chips
FILES=`ls -1l $1/$2/*_sl01.fits | ${P_GAWK} '($1 ~ /^-/) {print $NF}'`

for FILE in ${FILES}
do
  BASE=`basename ${FILE} _sl01.fits`

  RA=`get_key ${FILE} "RA"`
  if [ "${RA}" = "" ] || [ "${RA}" = "KEY_N/A" ]; then
      RA=0.0
  fi

  DEC=`get_key ${FILE} "DEC"`
  if [ "${DEC}" = "" ] || [ "${DEC}" = "KEY_N/A" ]; then
      DEC=0.0
  fi

  EQUINOX=`get_key ${FILE} "EQUINOX" | sed 's/[^0-9.]//g'`
  if [ "${EQUINOX}" = "" ] || [ "${EQUINOX}" = "KEY_N/A" ]; then
      EQUINOX=2000.
  fi
  
  AIRMASS=`get_key ${FILE} "HIERARCH ESO TEL AIRM START"`
  if [ "${AIRMASS}" = "" ] || [ "${AIRMASS}" = "KEY_N/A" ]; then
      AIRMASS=1.0
  fi

  ${P_REPLACEKEY} ${BASE}_sl01.fits "CRVAL1  = ${RA}" CRVAL1
  ${P_REPLACEKEY} ${BASE}_sl01.fits "CRVAL2  = ${DEC}" CRVAL2
  ${P_REPLACEKEY} ${BASE}_sl01.fits "EQUINOX = ${EQUINOX}" AMPLIST
  ${P_REPLACEKEY} ${BASE}_sl01.fits "AIRMASS = ${AIRMASS}" TSECA
  ${P_REPLACEKEY} ${BASE}_sl01.fits "FILTER  = 'i'" SUBSET

  ${P_REPLACEKEY} ${BASE}_sl02.fits "CRVAL1  = ${RA}" CRVAL1
  ${P_REPLACEKEY} ${BASE}_sl02.fits "CRVAL2  = ${DEC}" CRVAL2
  ${P_REPLACEKEY} ${BASE}_sl02.fits "EQUINOX = ${EQUINOX}" AMPLIST
  ${P_REPLACEKEY} ${BASE}_sl02.fits "AIRMASS = ${AIRMASS}" TSECA
  ${P_REPLACEKEY} ${BASE}_sl02.fits "FILTER  = 'z'" SUBSET

  ${P_REPLACEKEY} ${BASE}_sl03.fits "CRVAL1  = ${RA}" CRVAL1
  ${P_REPLACEKEY} ${BASE}_sl03.fits "CRVAL2  = ${DEC}" CRVAL2
  ${P_REPLACEKEY} ${BASE}_sl03.fits "EQUINOX = ${EQUINOX}" AMPLIST
  ${P_REPLACEKEY} ${BASE}_sl03.fits "AIRMASS = ${AIRMASS}" TSECA
  ${P_REPLACEKEY} ${BASE}_sl03.fits "FILTER  = 'g'" SUBSET

  ${P_REPLACEKEY} ${BASE}_sl04.fits "CRVAL1  = ${RA}" CRVAL1
  ${P_REPLACEKEY} ${BASE}_sl04.fits "CRVAL2  = ${DEC}" CRVAL2
  ${P_REPLACEKEY} ${BASE}_sl04.fits "EQUINOX = ${EQUINOX}" AMPLIST
  ${P_REPLACEKEY} ${BASE}_sl04.fits "AIRMASS = ${AIRMASS}" TSECA
  ${P_REPLACEKEY} ${BASE}_sl04.fits "FILTER  = 'r'" SUBSET

  ${P_FITSCUT} -i ${BASE}_sl01.fits -o ${BASE}_i_im1.fits -x   52 1098  4 2048
  ${P_FITSCUT} -i ${BASE}_sl01.fits -o ${BASE}_i_im2.fits -x 1106 2149  4 2048
  ${P_FITSCUT} -i ${BASE}_sl02.fits -o ${BASE}_z_im1.fits -x   52 1098 53 2097
  ${P_FITSCUT} -i ${BASE}_sl02.fits -o ${BASE}_z_im2.fits -x 1106 2149 53 2097
  ${P_FITSCUT} -i ${BASE}_sl03.fits -o ${BASE}_g_im1.fits -x   52 1098 53 2097
  ${P_FITSCUT} -i ${BASE}_sl03.fits -o ${BASE}_g_im2.fits -x 1106 2149 53 2097
  ${P_FITSCUT} -i ${BASE}_sl04.fits -o ${BASE}_r_im1.fits -x   52 1098  4 2048
  ${P_FITSCUT} -i ${BASE}_sl04.fits -o ${BASE}_r_im2.fits -x 1106 2149  4 2048
  \rm ${FILE} ${BASE}_sl0[0-4].fits
done

# now do the standard splitting
FILES=`ls -1l $1/$2/*_im[1,2].fits | ${P_GAWK} '($1 ~ /^-/) {print $NF}'`

for FILE in ${FILES}
do
  SUFFIX=`echo $FILE | ${P_GAWK} 'BEGIN {FS="."} {print "."$NF}'`
  BASE=`basename $FILE ${SUFFIX}`
  mv ${FILE} ${BASE}.fits
  FILE=${BASE}.fits

  RA=`get_key ${FILE} "CRVAL1"`

  DEC=`get_key ${FILE} "CRVAL2"`

  EXPTIME=`get_key ${FILE} "EXPTIME"`
  if [ "${EXPTIME}" = "" ] || [ "${EXPTIME}" = "KEY_N/A" ]; then
      EXPTIME=1.0
  fi

  OBJECT=`get_key ${FILE} "OBJECT"`
  if [ "${OBJECT}" = "" ] || [ "${OBJECT}" = "KEY_N/A" ]; then
      OBJECT=UNKNOWN
  fi

  MJD=`get_key ${FILE} "MJD-OBS"`
  if [ "${MJD}" = "" ] || [ "${MJD}" = "KEY_N/A" ]; then
      MJD=52000.0
  fi

  FILTER=`get_key ${FILE} "FILTER" | ${P_CLEANSTRING}`

  AIRMASS=`get_key ${FILE} "AIRMASS"`

  GABODSID=`${P_NIGHTID} -t 16:00:00 -d 31/12/1998 -m ${MJD} |\
      ${P_GAWK} ' ($1 ~ /Days/) {print $6}' | ${P_GAWK} 'BEGIN{ FS="."} {print $1}'`

  chip_id_tmp=`echo ${FILE} | ${P_GAWK} 'BEGIN{FS="_"} {print $NF}'`

  if [ "${chip_id_tmp}" = "im1.fits" ]; then
      chip=1
      REFPIXX=1050
      REFPIXY=1028
  else
      chip=2
      REFPIXX=-500
      REFPIXY=1028
  fi

  NCHIPS=1
  export NCHIPS
  ${P_FITSSPLIT_ECL} \
      -CRPIX1 "${REFPIXX}" \
      -CRPIX2 "${REFPIXY}" \
      -CRVAL1 ${RA} -CRVAL2 ${DEC} -EXPTIME ${EXPTIME} -AIRMASS ${AIRMASS} \
      -GABODSID ${GABODSID} -FILTER ${FILTER} -EQUINOX ${EQUINOX} -OBJECT ${OBJECT} \
      -CD11 "${PIXSCX}" \
      -CD12 "0" \
      -CD21 "0" \
      -CD22 "${PIXSCY}" \
      -M11 "0" \
      -M12 "1" \
      -M21 "1" \
      -M22 "0" \
      ${FILE}

  tmp=`get_key ${FILE} "DATE-OBS"`
  ${P_REPLACEKEY} ${BASE}_1.fits "DATE-OBS= ${tmp}" DUMMY2
  ${P_REPLACEKEY} ${BASE}_1.fits "IMAGEID = ${chip}" DUMMY3

  CD11=`echo ${PIXSCX} | ${P_GAWK} '{if ($1>0) print $1*(-1.); else print $1}'`
  CD22=`echo ${PIXSCY} | ${P_GAWK} '{if ($1<0) print $1*(-1.); else print $1}'`
  ${P_REPLACEKEY} ${BASE}_1.fits "CD1_1   = ${CD11}" CD1_1
  ${P_REPLACEKEY} ${BASE}_1.fits "CD2_2   = ${CD22}" CD2_2
  ${P_REPLACEKEY} ${BASE}_1.fits "CD1_2   = 0.0" CD1_2
  ${P_REPLACEKEY} ${BASE}_1.fits "CD2_1   = 0.0" CD2_1

  \rm ${FILE}
done

ls *im1_1.fits > ${TEMPDIR}/list_$$
cat ${TEMPDIR}/list_$$ |\
{
  while read FILE
  do
    base=`basename ${FILE} _im1_1.fits`
    mv ${FILE} ${base}_1.fits
  done
}

ls *im2_1.fits > ${TEMPDIR}/list_$$
cat ${TEMPDIR}/list_$$ |\
{
  while read FILE
  do
    base=`basename ${FILE} _im2_1.fits`
    mv ${FILE} ${base}_2.fits
    ${P_REPLACEKEY} ${base}_2.fits "CRPIX2  = 1" CRPIX2
  done
}

mv *_i_*.fits $1/$2_i/
mv *_z_*.fits $1/$2_z/
mv *_g_*.fits $1/$2_g/
mv *_r_*.fits $1/$2_r/
