#!/bin/bash -xv

# ***************************************
# ***** INSTRUMENT splitting script *****
# *****      created by THELI       *****
# ***************************************

# splits VISIR@VLT images, returns the average of each chopping position

# $1: main directory
# $2: science directory

INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

. bash.include


# rename the images to the value of a FITS key (if requested)

if [ "${V_PRE_RENAME_CHECKED}" = "1" ]; then

  cd /$1/$2

  p_ls > ${TEMPDIR}/fitskey_$$

  cat ${TEMPDIR}/fitskey_$$ |\
  {
    while read FILE
    do
      newname=`get_key ${FILE} ${V_RENAME_FITSKEY}`
      if [ "${newname}" != "KEY_N/A" ] && [ "${newname}" != "" ]; then
         mv ${FILE} ${newname}
      else
         echo ERROR: Could not find ${V_RENAME_FITSKEY} in the FITS header
      fi
    done
  }
fi



DIR=`pwd`

cd /$1/$2

mkdir ORIGINALS

CUBES=`ls -1l $1/$2/ | ${P_GAWK} '($1 ~ /^-/) {print $NF}'`

for CUBE in ${CUBES}
do
  SUFFIX=`echo $CUBE | ${P_GAWK} 'BEGIN {FS="."} {print "."$NF}'`
  BASE=`basename $CUBE $SUFFIX`
  BASE=`basename ${CUBE} .fits`
  nod=`get_key ${FILE} "HIERARCH ESO SEQ NODPOS"`
  mv ${CUBE} ${BASE}_${nod}.fits
  ${P_SPLITVISIRFITSCUBE} -i ${BASE}_${nod}.fits -a
  mv ${BASE}_${nod}.fits ORIGINALS/${BASE}${SUFFIX}
done

cd ${DIR}

FILES=`ls -1l $1/$2/ | ${P_GAWK} '($1 ~ /^-/) {print $NF}'`

cd /$1/$2

for FILE in ${FILES}
do
  SUFFIX=`echo $FILE | ${P_GAWK} 'BEGIN {FS="."} {print "."$NF}'`
  BASE=`basename $FILE $SUFFIX`
  mv ${FILE} ${BASE}.fits
  FILE=${BASE}.fits

  RA=`get_key ${FILE} "RA"`
  if [ "${RA}" = "" ] || [ "${RA}" = "KEY_N/A" ]; then
      RA=0.0
  fi

  DEC=`get_key ${FILE} "DEC"`
  if [ "${DEC}" = "" ] || [ "${DEC}" = "KEY_N/A" ]; then
      DEC=0.0
  fi

  EXPTIME=`get_key ${FILE} "EXPTIME"`
  NDIT=`get_key ${FILE} "HIERARCH ESO DET NDIT"`
  EXPTIME=`echo ${EXPTIME} ${NDIT} | ${P_GAWK} '{print $1*$2}'`
  if [ "${EXPTIME}" = "" ] || [ "${EXPTIME}" = "KEY_N/A" ]; then
      EXPTIME=1.0
  fi

  OBJECT=`get_key ${FILE} "OBJECT"`
  if [ "${OBJECT}" = "" ] || [ "${OBJECT}" = "KEY_N/A" ]; then
      OBJECT=UNKNOWN
  fi

  EQUINOX=`get_key ${FILE} "EQUINOX" | sed 's/[^0-9.]//g'`
  if [ "${EQUINOX}" = "" ] || [ "${EQUINOX}" = "KEY_N/A" ]; then
      EQUINOX=2000.0
  fi

  MJD=`get_key ${FILE} "MJD-OBS"`
  if [ "${MJD}" = "" ] || [ "${MJD}" = "KEY_N/A" ]; then
      MJD=52000.0
  fi

  FILTER=`get_key ${FILE} "HIERARCH ESO INS FILT1 NAME" | ${P_CLEANSTRING}`
  if [ "${FILTER}" = "" ] || [ "${FILTER}" = "KEY_N/A" ]; then
      FILTER=UNKNOWN
  fi

  LST=`get_key ${FILE} "LST"`
  if [ "${LST}" = "" ] || [ "${LST}" = "KEY_N/A" ]; then
      LST=0.0
  fi

  AIRMASS=`get_key ${FILE} "HIERARCH ESO TEL AIRM START"`
  if [ "${AIRMASS}" = "" ] || [ "${AIRMASS}" = "KEY_N/A" ]; then
      AIRMASS=1.0
  fi

  GABODSID=`${P_NIGHTID} -t 16:00:00 -d 31/12/1998 -m ${MJD} |\
      ${P_GAWK} ' ($1 ~ /Days/) {print $6}' | ${P_GAWK} 'BEGIN{ FS="."} {print $1}'`

  cd11=`get_key ${FILE} "CD1_1"`
  cd12=`get_key ${FILE} "CD1_2"`
  cd21=`get_key ${FILE} "CD2_1"`
  cd22=`get_key ${FILE} "CD2_2"`
  crpix1=`get_key ${FILE} "CRPIX1"`
  crpix2=`get_key ${FILE} "CRPIX2"`

  ${P_FITSSPLIT_ECL} \
      -CRPIX1 "${crpix1}" \
      -CRPIX2 "${crpix2}" \
      -CD11 "${cd11}" \
      -CD12 "${cd12}" \
      -CD21 "${cd21}" \
      -CD22 "${cd22}" \
      -CRVAL1 ${RA} -CRVAL2 ${DEC} -EXPTIME ${EXPTIME} -AIRMASS ${AIRMASS} \
      -GABODSID ${GABODSID} -FILTER ${FILTER} -EQUINOX ${EQUINOX} -OBJECT ${OBJECT} \
      -M11 "1" \
      -M12 "0" \
      -M21 "0" \
      -M22 "1" \
      ${FILE}

  k=1
  while [ "${k}" -le "${NCHIPS}" ]
  do
    tmp=`get_key ${FILE} "HIERARCH ESO DET CHOP FREQ"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "CHOPFREQ= ${tmp}" DUMMY1
    tmp=`get_key ${FILE} "HIERARCH ESO DET CHOP NCYCLES"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "CNCYCLES= ${tmp}" DUMMY2
    tmp=`get_key ${FILE} "HIERARCH ESO DET NDIT"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "NDIT    = ${tmp}" DUMMY3
    tmp=`get_key ${FILE} "HIERARCH ESO INS EXTS ACT"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "SRC_TEMP= ${tmp}" DUMMY4
    tmp=`get_key ${FILE} "HIERARCH ESO INS TEMP1 VAL"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "DETTEMP1= ${tmp}" DUMMY5
    tmp=`get_key ${FILE} "HIERARCH ESO INS TEMP2 VAL"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "DETTEMP2= ${tmp}" DUMMY6
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ CUMOFFSETX"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "CUMXOFF = ${tmp}" DUMMY7
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ CUMOFFSETY"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "CUMYOFF = ${tmp}" DUMMY8
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ JITTER WIDTH"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "JITTER  = ${tmp}" DUMMY9
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ NODPOS"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "NODPOS  = ${tmp}" DUMMY10
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ RELOFFSETX"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "RELXOFF = ${tmp}" DUMMY11
    tmp=`get_key ${FILE} "HIERARCH ESO SEQ RELOFFSETY"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "RELYOFF = ${tmp}" DUMMY12
    tmp=`get_key ${FILE} "DATE-OBS"`
    ${P_REPLACEKEY} ${BASE}_${k}.fits "DATE-OBS = ${tmp}" DUMMY13
    k=$(( $k + 1 ))
  done

  \rm ${FILE} ORIGINALS/${BASE}${SUFFIX}

done
