#!/bin/bash -xv

# this script copies the original superflat and illum and fringe
# into ORIG_SUPERFLAT, and replaces them with the superflat from
# another directory (mostly offtarget or empty field)

# $1: main dir
# $2: science dir
# $3: offtarget dir
# $4: chips to be processed

# preliminary work:
INSTDIR=instruments_professional
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=instruments_commercial
fi
if [ ! -f ${INSTDIR}/${INSTRUMENT}.ini ]; then
    INSTDIR=~/.theli/instruments_user/
fi
. ${INSTDIR}/${INSTRUMENT:?}.ini

mkdir $1/$2/ORIG_SUPERFLAT

for CHIP in $4
do

  if [ ! -f /$1/$2/$2_${CHIP}.fits ]; then
      touch /$1/$2/$2_${CHIP}.fits
  fi

  FILE=`ls /$1/$2/$2_${CHIP}.fits`
  OFFFILE=`ls /$1/$3/$3_${CHIP}.fits`

  if [ -L ${FILE} ]; then
    LINK=`${P_READLINK} ${FILE}`
    BASE=`basename ${LINK} .fits`
    DIR=`dirname ${LINK}`

    OFFLINK=`${P_READLINK} ${OFFFILE}`
    OFFBASE=`basename ${OFFLINK} .fits`
    OFFDIR=`dirname ${OFFLINK}`

    mkdir ${DIR}/ORIG_SUPERFLAT

    # move the superflats in the physical dir to ORIG_SUPERFLAT
    test -e ${DIR}/${BASE}.fits        && mv ${DIR}/${BASE}.fits ${DIR}/ORIG_SUPERFLAT
    test -e ${DIR}/${BASE}_illum.fits  && mv ${DIR}/${BASE}_illum.fits ${DIR}/ORIG_SUPERFLAT
    test -e ${DIR}/${BASE}_fringe.fits && mv ${DIR}/${BASE}_fringe.fits ${DIR}/ORIG_SUPERFLAT

    # link the superflats in the physical ORIG to the apparent ORIG
    ln -s ${DIR}/ORIG_SUPERFLAT/${BASE}.fits \
	/$1/$2/ORIG_SUPERFLAT/${BASE}.fits
    ln -s ${DIR}/ORIG_SUPERFLAT/${BASE}_illum.fits \
	/$1/$2/ORIG_SUPERFLAT/${BASE}_illum.fits
    ln -s ${DIR}/ORIG_SUPERFLAT/${BASE}_fringe.fits \
	/$1/$2/ORIG_SUPERFLAT/${BASE}_fringe.fits

    # remove the apparent superflats
    test -e /$1/$2/${BASE}.fits        && rm /$1/$2/${BASE}.fits 
    test -e /$1/$2/${BASE}_illum.fits  && rm /$1/$2/${BASE}_illum.fits 
    test -e /$1/$2/${BASE}_fringe.fits && rm /$1/$2/${BASE}_fringe.fits

    # copy the physical offtarget superflats to the physical science dir
    cp ${OFFDIR}/${OFFBASE}.fits ${DIR}/${BASE}.fits
    test -e ${OFFDIR}/${OFFBASE}_illum.fits  && cp ${OFFDIR}/${OFFBASE}_illum.fits ${DIR}/${BASE}_illum.fits
    test -e ${OFFDIR}/${OFFBASE}_fringe.fits && cp ${OFFDIR}/${OFFBASE}_fringe.fits ${DIR}/${BASE}_fringe.fits

    # create links to the physical offtarget superflats
    ln -s ${DIR}/${BASE}.fits /$1/$2/${BASE}.fits
    ln -s ${DIR}/${BASE}_illum.fits /$1/$2/${BASE}_illum.fits
    ln -s ${DIR}/${BASE}_fringe.fits /$1/$2/${BASE}_fringe.fits

  else
    BASE=`basename ${FILE} .fits`
    DIR="/$1/$2"
    OFFBASE=`basename ${OFFFILE} .fits`
    OFFDIR="/$1/$3"
    mv ${DIR}/${BASE}.fits ${DIR}/ORIG_SUPERFLAT
    test -e ${DIR}/${BASE}_illum.fits  && mv ${DIR}/${BASE}_illum.fits ${DIR}/ORIG_SUPERFLAT
    test -e ${DIR}/${BASE}_fringe.fits && mv ${DIR}/${BASE}_fringe.fits ${DIR}/ORIG_SUPERFLAT
    cp ${OFFDIR}/${OFFBASE}.fits ${DIR}/${BASE}.fits
    test -e ${OFFDIR}/${OFFBASE}_illum.fits  && cp ${OFFDIR}/${OFFBASE}_illum.fits ${DIR}/${BASE}_illum.fits
    test -e ${OFFDIR}/${OFFBASE}_fringe.fits && cp ${OFFDIR}/${OFFBASE}_fringe.fits ${DIR}/${BASE}_fringe.fits
  fi

done
