 /*
 				misc.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	SExtractor
*
*	Author:		E.BERTIN, IAP/ESO
*
*	Contents:	miscellaneous functions.
*
*	Last modify:	31/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*
   07.03.2005:
   I included code for the get_basename function that returns the
   basename for a file containing the complete path.

   22.08.2006:
   - I included string.h to avoid compiler warnings concerning 
     strrchr
   - I changed the code for performing byte swapping (alias problems
     under gcc4.1? Changes were done after recommendation in the TERAPIX
     forums).
 */

#include	<ctype.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"


/********************************* error ************************************/
/*
I hope it will never be used!
*/
void	error(int num, char *msg1, char *msg2)
  {
  fprintf(stderr, "\n> %s%s\n\n",msg1,msg2);
  exit(num);
  }


/******************************* cistrcmp ***********************************/
/*
case-insensitive strcmp.
*/
int	cistrcmp(char *cs, char *ct, int mode)

  {
   int	diff;

  if (mode)
    {
    while (*cs && *ct)
      if (diff=tolower((int)*(cs++))-tolower((int)*(ct++)))
        return diff;
    }
  else
    {
    while (*cs || *ct)
      if (diff=tolower((int)*(cs++))-tolower((int)*(ct++)))
        return diff;
    }

  return 0;
  }


/******************************** hmedian ***********************************/
/*
Median using Heapsort algorithm (for float arrays) (based on Num.Rec algo.).
Warning: changes the order of data!
*/
float	hmedian(float *ra, int n)

  {
   int		l, j, ir, i;
   float	rra;


  if (n<2)
    return *ra;
  ra--;
  for (l = ((ir=n)>>1)+1;;)
    {
    if (l>1)
      rra = ra[--l];
    else
      {
      rra = ra[ir];
      ra[ir] = ra[1];
      if (--ir == 1)
        {
        ra[1] = rra;
        return n&1? ra[n/2+1] : (float)((ra[n/2]+ra[n/2+1])/2.0);
        }
      }
    for (j = (i=l)<<1; j <= ir;)
      {
      if (j < ir && ra[j] < ra[j+1])
        ++j;
      if (rra < ra[j])
        {
        ra[i] = ra[j];
        j += (i=j);
        }
      else
        j = ir + 1;
      }
    ra[i] = rra;
    }

/* (the 'return' is inside the loop!!) */
  }


/********************************* warning **********************************/
/*
Print a warning message on screen.
*/
void	warning(char *msg1, char *msg2)
  {
  fprintf(OUTPUT, "\n> WARNING: %s%s\n\n",msg1,msg2);
  return;
  }


/******************************* swapbytes **********************************/
/*
Swap bytes for doubles, longs and shorts (for DEC machines or PC for inst.).
*/
void	swapbytes(void *ptr, int nb, int n)
  {
   char	*cp;
   char t;
   int	j;

  cp = (char *)ptr;

  if (nb==4)
    {
    for (j=n; j--; cp+=4)
      {
         t = cp[0];
         cp[0] = cp[3];
         cp[3] = t;
         t = cp[1];
         cp[1] = cp[2];
         cp[2] = t;
      }
    return;
    }

  if (nb==2)
    {
    for (j=n; j--; cp+=2)
      {
         t = cp[0];
         cp[0] = cp[1];
         cp[1] = t;      
      }
    return;
    }

  if (nb&1)
    return;

  if (nb==8)
    {
    for (j=n; j--; cp+=8)
      {
         t = cp[0];
         cp[0] = cp[7];
         cp[7] = t;
         t = cp[1];
         cp[1] = cp[6];
         cp[6] = t;
         t = cp[2];
         cp[2] = cp[5];
         cp[5] = t;
         t = cp[3];
         cp[3] = cp[4];
         cp[4] = t;
      }
    return;
    }

  error(EXIT_FAILURE, "*Internal Error*: Unknown size in ", "swapbytes()");

  return;
  }

/******************************* get_basename *******************************/
/*
return 'basename' for a filename including complete path
*/
char * get_basename(const char *filename)
{
  char *p ;
  p = strrchr (filename, '/');
  return p ? p + 1 : (char *) filename;
}
